/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Sep 28, 2004, 3:40:04 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.functest.calcsteps;

import isac.bridge.CalcIterator;
import isac.bridge.CalcTree;
import isac.bridge.MathEngine;
import isac.bridge.MockIToUser;
import isac.browserdialog.MockProblemBrowserFrame;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.useractions.IUserAction;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnCalcElement;
import isac.useractions.UserActionOnIterator;
import isac.users.User;
import isac.util.CalcChanged;
import isac.util.Formalization;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.util.parser.FormalizationDigest;
import isac.wsdialog.IWorksheetDialog;

import java.rmi.RemoteException;
import java.util.Vector;

import junit.framework.TestCase;

/**
 * The learner solves an example stepwise supported by ISAC (the CalcHead is
 * assumed to be completed already): He requests ISAC to do some steps, to apply
 * certain Tactics (eventually selected from a list of Tactics requested from
 * ISAC), he inputs the next formula. He also redoes parts of the example by
 * setting the ActiveFormula to a Formula somewhere in the calculation and
 * applies another Tactic or inputs another Formula.
 * 
 * @see <isac.functest.TestModel>for input of items to the Model on the CalcHead
 *      and requesting related help from ISAC.
 * @see <isac.functest.TestSpecify>for specifying theories, problems and methods
 *      (via CalcHead) in the math knowledge base.
 * @see <isac.functest.TestAlgebraSysten>for automated solving.
 * 
 * @author Walther Neuper Sep 28, 2004, 3:40:04 PM
 * @version 1.0
 */
public class TestSolveStepwise extends TestCase {

    private IToCalc ct_;

    private ICalcIterator it_;

    private MockIToUser user_;

    private CalcChanged cce_;

    // setup "solve(x + 1 = 2, x)" as seen from the WorkSheet
    // (ie. behind the WorkSheetDialog)
    protected void setUp() throws Exception {
        super.setUp();
        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization formalization = fd.parseFormalization((String) xmlstr);

        MathEngine.init("localhost");
        MathEngine math_engine = MathEngine.getMathEngine();

        //this would be preferable, but only works until (#)
        //
        //XSessionManager session_manager = XSessionManager.getInstance();
        //String session_id = (String)
        // session_manager.login("x","x").elementAt(0);
        //
        //String wsd_id = session_manager.openWorksheetDialog(session_id);
        //IWorksheetDialog wsd = session_manager.getWorksheetDialog(wsd_id);
        //
        //this would include registering a WorkSheet as listener
        // + again include a WindowApplication ???.................(#)
        //CalcTree ct = wsd.getCalcTree(0, formalization,
        //      IWorksheetDialog.STARTFROM_EXAMPLE);
        //        ct.startCalculate();
        //        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        //
        //ie. one would need an MockIWorksheetDialog !!!!!!!!!!!!?
        // or even a MockSessionManager !!!

        ct_ = math_engine.getCalcTree(formalization);
        it_ = ct_.iterator();
        user_ = new MockIToUser();
        ct_.addDataChangeListener(user_);
        ct_.completeCalcHead();
    }

    /**
     * calculate "solve(x + 1 = 2, x)
     */
    public void test1Step1App1Repl() {
        System.out
                .println("---BEGIN isac.functest.TestSolveStepwise#test1Step1App1Repl");
        Vector formulae = null;

        //######################### the button <NEXT> for "x + 1 = 2":
        try {
            ct_.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        //which results in this ...
        cce_ = user_.getCalcChangedEvent();
        try {
            assertEquals("unchanged", cce_.getLastUnchangedFormula().toSMLString(),
                    "([],Met)");
            assertEquals("deleted", cce_.getLastDeletedFormula().toSMLString(),
                "([],Met)");
            assertEquals("generated", cce_.getLastGeneratedFormula().toSMLString(),
                "([1],Frm)");
        } catch (RemoteException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        try {
            formulae = (cce_.getLastUnchangedFormula()).getFormulaeFromTo(cce_
                    .getLastGeneratedFormula(), new Integer(0), false);
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }
        assertEquals("Vector with 1st form", formulae.size(), 1);
        assertEquals("1st form", ((CalcFormula) (formulae.firstElement()))
                .toSMLString(), "x + 1 = 2");

        //######################### input "x - 1 = 0" below "x + 1 = 2":
        //actually the WorkSheet finally sends:
        //dialog_.notifyUserAction(new UserActionOnCalcElement(
        //	  IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
        //          edited_formula_));
        CalcFormula in = new CalcFormula();
        in.setText("x - 1 = 0");
        try {
            ct_.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        } catch (RemoteException e2) {
            e2.printStackTrace();
        }
        cce_ = user_.getCalcChangedEvent();
        try {
            assertEquals("unchanged", cce_.getLastUnchangedFormula().toSMLString(),
                    "([1],Frm)");
            assertEquals("deleted", cce_.getLastDeletedFormula().toSMLString(),
                "([1],Frm)");
            assertEquals("generated", cce_.getLastGeneratedFormula().toSMLString(),
                "([2],Res)");
        } catch (RemoteException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        //get the formulae on the _same_ level: level = new Integer(0) !
        try {
            formulae = (cce_.getLastUnchangedFormula()).getFormulaeFromTo(cce_
                    .getLastGeneratedFormula(), new Integer(0), false);
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }
        assertEquals("Vector with 2nd form", formulae.size(), 2);
        assertEquals("1st form", ((CalcFormula) (formulae.elementAt(0)))
                .toSMLString(), "x + 1 + -1 * 2 = 0");
        assertEquals("2nd form", ((CalcFormula) (formulae.elementAt(1)))
                .toSMLString(), "x - 1 = 0");
        //however, there have more formulae been generated:
        // level = new Integer(99) !
        //(but show them only on request of <intermediate steps> !!!)
        try {
            formulae = (cce_.getLastUnchangedFormula()).getFormulaeFromTo(cce_
                    .getLastGeneratedFormula(), new Integer(99), false);
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }
        assertEquals("Vector with all forms", formulae.size(), 12);

        //######################### input "x = 1" once more below "x + 1 = 2":
        //actually the WorkSheet finally sends:
        //dialog_.notifyUserAction(new UserActionOnCalcElement(
        //	  IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
        //          edited_formula_));
        Position pos = new Position();//([2],Res)
        pos.addInt("1");
        pos.setKind("Res");
        it_ = new CalcIterator((CalcTree) ct_, pos);
        try {
            ct_.moveActiveFormula(it_);
        } catch (RemoteException e3) {
            e3.printStackTrace();
        }
        in = new CalcFormula();
        in.setText("x = 1");
        try {
            ct_.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        } catch (RemoteException e4) {
            e4.printStackTrace();
        }
        cce_ = user_.getCalcChangedEvent();

        try {
            assertEquals("unchanged", cce_.getLastUnchangedFormula().toSMLString(),
                    "([1],Frm)");
            assertEquals("deleted", cce_.getLastDeletedFormula().toSMLString(),
                    "([2],Res)");
            assertEquals("generated", cce_.getLastGeneratedFormula().toSMLString(),
                    "([2,2],Res)");
        } catch (RemoteException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        //get the _minimal_ number of formulae: level = new Integer(0) !
        try {
            formulae = (cce_.getLastUnchangedFormula()).getFormulaeFromTo(cce_
                    .getLastGeneratedFormula(), new Integer(0), false);
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }
        assertEquals("Vector with 3rd form", formulae.size(), 5);

        CalcFormula f1 = (CalcFormula) (formulae.elementAt(0));
        CalcFormula f5 = (CalcFormula) (formulae.elementAt(4));
        assertEquals("1st form", f1.toSMLString(), "1 + x = 2");
        assertEquals("last form", f5.toSMLString(), "x = 1");
        //and observe the level required for inserting into the WS-tree:
        assertEquals("level of 1st form", f1.getPosition().getLevel(), 1);
        assertEquals("level of last form", f5.getPosition().getLevel(), 2);

        System.out
                .println("---END isac.functest.TestSolveStepwise#test1Step1App1Repl");
    }
}